@@ -12,7 +12,7 @@
 //! and is used to actually do things.
 
 use crate::{
-    buffer, format, image, memory,
+    buffer, display, external_memory, format, image, memory,
     memory::{Requirements, Segment},
     pass,
     pool::CommandPoolCreateFlags,
@@ -555,7 +555,7 @@ pub trait Device<B: Backend>: fmt::Debug + Any + Send + Sync {
     /// [queue submission][crate::queue::Queue::submit] command.
     ///
     /// Fences **can** be unsignaled on the host with
-    /// [`reset_fences`][Device::reset_fences].
+    /// [`reset_fence`][Device::reset_fence].
     ///
     /// Fences **can** be waited on by the host with the
     /// [`wait_for_fences`][Device::wait_for_fences] command.
@@ -714,6 +714,165 @@ pub trait Device<B: Backend>: fmt::Debug + Any + Send + Sync {
     /// validation layers that can print a friendly name when referring to objects in error messages
     unsafe fn set_pipeline_layout_name(&self, pipeline_layout: &mut B::PipelineLayout, name: &str);
 
+    /// Control the power state of the provided display
+    unsafe fn set_display_power_state(
+        &self,
+        display: &display::Display<B>,
+        power_state: &display::control::PowerState,
+    ) -> Result<(), display::control::DisplayControlError>;
+
+    /// Register device event
+    unsafe fn register_device_event(
+        &self,
+        device_event: &display::control::DeviceEvent,
+        fence: &mut B::Fence,
+    ) -> Result<(), display::control::DisplayControlError>;
+
+    /// Register display event
+    unsafe fn register_display_event(
+        &self,
+        display: &display::Display<B>,
+        display_event: &display::control::DisplayEvent,
+        fence: &mut B::Fence,
+    ) -> Result<(), display::control::DisplayControlError>;
+
+    /// Create, allocate and bind a buffer that can be exported.
+    /// # Arguments
+    ///
+    /// * `external_memory_type_flags` - the external memory types the buffer will be valid to be used.
+    /// * `usage` - the usage of the buffer.
+    /// * `sparse` - the sparse flags of the buffer.
+    /// * `type_mask` - a memory type mask containing all the desired memory type ids.
+    /// * `size` - the size of the buffer.
+    /// # Errors
+    ///
+    /// - Returns `OutOfMemory` if the implementation goes out of memory during the operation.
+    /// - Returns `TooManyObjects` if the implementation can allocate buffers no more.
+    /// - Returns `NoValidMemoryTypeId` if the no one of the desired memory type id is valid for the implementation.
+    /// - Returns `InvalidExternalHandle` if the requested external memory type is invalid for the implementation.
+    ///
+    unsafe fn create_allocate_external_buffer(
+        &self,
+        external_memory_type_flags: external_memory::ExternalBufferMemoryType,
+        usage: buffer::Usage,
+        sparse: memory::SparseFlags,
+        type_mask: u32,
+        size: u64,
+    ) -> Result<(B::Buffer, B::Memory), external_memory::ExternalResourceError>;
+
+    /// Import external memory as binded buffer and memory.
+    /// # Arguments
+    ///
+    /// * `external_memory` - the external memory types the buffer will be valid to be used.
+    /// * `usage` - the usage of the buffer.
+    /// * `sparse` - the sparse flags of the buffer.
+    /// * `type_mask` - a memory type mask containing all the desired memory type ids.
+    /// * `size` - the size of the buffer.
+    /// # Errors
+    ///
+    /// - Returns `OutOfMemory` if the implementation goes out of memory during the operation.
+    /// - Returns `TooManyObjects` if the implementation can allocate buffers no more.
+    /// - Returns `NoValidMemoryTypeId` if the no one of the desired memory type id is valid for the implementation.
+    /// - Returns `InvalidExternalHandle` if the requested external memory type is invalid for the implementation.
+    ///
+    unsafe fn import_external_buffer(
+        &self,
+        external_memory: external_memory::ExternalBufferMemory,
+        usage: buffer::Usage,
+        sparse: memory::SparseFlags,
+        type_mask: u32,
+        size: u64,
+    ) -> Result<(B::Buffer, B::Memory), external_memory::ExternalResourceError>;
+
+    /// Create, allocate and bind an image that can be exported.
+    /// # Arguments
+    ///
+    /// * `external_memory` - the external memory types the image will be valid to be used.
+    /// * `kind` - the image kind.
+    /// * `mip_levels` - the mip levels of the image.
+    /// * `format` - the format of the image.
+    /// * `tiling` - the tiling mode of the image.
+    /// * `dimensions` - the dimensions of the image.
+    /// * `usage` - the usage of the image.
+    /// * `sparse` - the sparse flags of the image.
+    /// * `view_caps` - the view capabilities of the image.
+    /// * `type_mask` - a memory type mask containing all the desired memory type ids.
+    /// # Errors
+    ///
+    /// - Returns `OutOfMemory` if the implementation goes out of memory during the operation.
+    /// - Returns `TooManyObjects` if the implementation can allocate images no more.
+    /// - Returns `NoValidMemoryTypeId` if the no one of the desired memory type id is valid for the implementation.
+    /// - Returns `InvalidExternalHandle` if the requested external memory type is invalid for the implementation.
+    ///
+    unsafe fn create_allocate_external_image(
+        &self,
+        external_memory_type: external_memory::ExternalImageMemoryType,
+        kind: image::Kind,
+        mip_levels: image::Level,
+        format: format::Format,
+        tiling: image::Tiling,
+        usage: image::Usage,
+        sparse: memory::SparseFlags,
+        view_caps: image::ViewCapabilities,
+        type_mask: u32,
+    ) -> Result<(B::Image, B::Memory), external_memory::ExternalResourceError>;
+
+    /// Import external memory as binded image and memory.
+    /// # Arguments
+    ///
+    /// * `external_memory` - the external memory types the image will be valid to be used.
+    /// * `kind` - the image kind.
+    /// * `mip_levels` - the mip levels of the image.
+    /// * `format` - the format of the image.
+    /// * `tiling` - the tiling mode of the image.
+    /// * `dimensions` - the dimensions of the image.
+    /// * `usage` - the usage of the image.
+    /// * `sparse` - the sparse flags of the image.
+    /// * `view_caps` - the view capabilities of the image.
+    /// * `type_mask` - a memory type mask containing all the desired memory type ids.
+    /// # Errors
+    ///
+    /// - Returns `OutOfMemory` if the implementation goes out of memory during the operation.
+    /// - Returns `TooManyObjects` if the implementation can allocate images no more.
+    /// - Returns `NoValidMemoryTypeId` if the no one of the desired memory type id is valid for the implementation.
+    /// - Returns `InvalidExternalHandle` if the requested external memory type is invalid for the implementation.
+    ///
+    unsafe fn import_external_image(
+        &self,
+        external_memory: external_memory::ExternalImageMemory,
+        kind: image::Kind,
+        mip_levels: image::Level,
+        format: format::Format,
+        tiling: image::Tiling,
+        usage: image::Usage,
+        sparse: memory::SparseFlags,
+        view_caps: image::ViewCapabilities,
+        type_mask: u32,
+    ) -> Result<(B::Image, B::Memory), external_memory::ExternalResourceError>;
+
+    /// Export memory as os type (Fd, Handle or Ptr) based on the requested external memory type.
+    /// # Arguments
+    ///
+    /// * `external_memory_type` - the external memory type the memory will be exported to.
+    /// * `memory` - the memory object.
+    /// # Errors
+    ///
+    /// - Returns `OutOfMemory` if the implementation goes out of memory during the operation.
+    /// - Returns `TooManyObjects` if the implementation can allocate images no more.
+    /// - Returns `InvalidExternalHandle` if the requested external memory type is invalid for the implementation.
+    ///
+    unsafe fn export_memory(
+        &self,
+        external_memory_type: external_memory::ExternalMemoryType,
+        memory: &B::Memory,
+    ) -> Result<external_memory::PlatformMemory, external_memory::ExternalMemoryExportError>;
+
+    /// Retrieve the underlying drm format modifier from an image, if any.
+    /// # Arguments
+    ///
+    /// * `image` - the image object.
+    unsafe fn drm_format_modifier(&self, image: &B::Image) -> Option<format::DrmModifier>;
+
     /// Starts frame capture.
     fn start_capture(&self);
 
