@@ -89,8 +89,11 @@ pub enum SwapchainError {
     /// Window in use
     #[error("Window is in use")]
     WindowInUse,
+    /// Accecssing the underlying NSView from wrong thread https://github.com/gfx-rs/gfx/issues/3704
+    #[error("Accecssing NSView from wrong thread")]
+    WrongThread,
     /// Unknown error.
-    #[error("Swapchain can't be created for an unlknown reason")]
+    #[error("Swapchain can't be created for an unknown reason")]
     Unknown,
 }
 
@@ -115,6 +118,21 @@ impl From<image::Extent> for Extent2D {
     }
 }
 
+impl From<(image::Size, image::Size)> for Extent2D {
+    fn from(tuple: (image::Size, image::Size)) -> Self {
+        Extent2D {
+            width: tuple.0,
+            height: tuple.1,
+        }
+    }
+}
+
+impl From<Extent2D> for (image::Size, image::Size) {
+    fn from(extent: Extent2D) -> Self {
+        (extent.width, extent.height)
+    }
+}
+
 impl Extent2D {
     /// Convert into a regular image extent.
     pub fn to_extent(&self) -> image::Extent {
@@ -126,6 +144,26 @@ impl Extent2D {
     }
 }
 
+/// An offset describes the position of a rectangle, such as
+/// a window or texture.
+#[derive(Clone, Copy, Debug, PartialEq, PartialOrd)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+pub struct Offset2D {
+    /// X
+    pub x: image::TexelCoordinate,
+    /// Y
+    pub y: image::TexelCoordinate,
+}
+
+impl From<(image::TexelCoordinate, image::TexelCoordinate)> for Offset2D {
+    fn from(tuple: (image::TexelCoordinate, image::TexelCoordinate)) -> Self {
+        Offset2D {
+            x: tuple.0,
+            y: tuple.1,
+        }
+    }
+}
+
 /// Describes information about what a `Surface`'s properties are.
 /// Fetch this with [Surface::capabilities].
 #[derive(Debug, Clone)]
